import { resolveHTTPResponse } from '@trpc/server/http';
/**
 * Create a SvelteKit handle function for tRPC requests.
 *
 * If you want to use it in conjunction with other SvelteKit handles,
 * consider [the sequence helper function](https://kit.svelte.dev/docs/modules#sveltejs-kit-hooks).
 * @see https://kit.svelte.dev/docs/hooks
 */
export function createTRPCHandle({ router, url = '/trpc', createContext, responseMeta, onError }) {
    return async ({ event, resolve }) => {
        if (event.url.pathname.startsWith(url + '/')) {
            const request = event.request;
            const req = {
                method: request.method,
                headers: request.headers,
                query: event.url.searchParams,
                body: await request.text()
            };
            // Using the default `event.setHeaders` and `event.cookies` will not work
            // as the event in not resolved by SvelteKit. Instead, we "proxy" the access
            // to the headers and cookies, so that we can set them later.
            const headersProxy = {};
            const originalCookiesSet = event.cookies.set;
            const originalCookiesDelete = event.cookies.delete;
            const newCookiesNames = [];
            const deleteCookiesNames = [];
            event.setHeaders = (headers) => {
                for (const [key, value] of Object.entries(headers)) {
                    headersProxy[key] = value;
                }
            };
            event.cookies.set = (name, value, opts) => {
                newCookiesNames.push(name);
                if (deleteCookiesNames.includes(name)) {
                    deleteCookiesNames.splice(deleteCookiesNames.indexOf(name), 1);
                }
                originalCookiesSet(name, value, opts);
            };
            event.cookies.delete = (name) => {
                deleteCookiesNames.push(name);
                if (newCookiesNames.includes(name)) {
                    newCookiesNames.splice(newCookiesNames.indexOf(name), 1);
                }
                originalCookiesDelete(name);
            };
            const httpResponse = await resolveHTTPResponse({
                router,
                req,
                path: event.url.pathname.substring(url.length + 1),
                createContext: async () => createContext?.(event),
                responseMeta,
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
                onError: onError
            });
            const { status, headers, body } = httpResponse;
            // Set headers and cookies that were set using SvelteKit's `event.setHeaders` and `event.cookies.set`.
            for (const [key, value] of Object.entries(headersProxy)) {
                headers[key] = value;
            }
            const cookies = event.cookies.getAll().filter((cookie) => {
                // Only pick new cookies
                if (!newCookiesNames.includes(cookie.name))
                    return false;
                // Don't pick cookies that were deleted
                if (deleteCookiesNames.includes(cookie.name))
                    return false;
                return true;
            });
            if (cookies.length > 0) {
                headers['Set-Cookie'] = cookies
                    .map((cookie) => `${cookie.name}=${cookie.value}`)
                    .join('; ');
            }
            return new Response(body, { status, headers });
        }
        return resolve(event);
    };
}
